/**
 * (C) Copyright IBM Corp. 2021.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/// <reference types="node" />
import { IncomingHttpHeaders, OutgoingHttpHeaders } from 'http';
import { BaseService, UserOptions } from 'ibm-cloud-sdk-core';
/**
 * SQL Query is a stateless service for analyzing rectangular data stored in IBM Cloud Object Store using ANSI SQL.
 */
declare class SqlV2 extends BaseService {
    static DEFAULT_SERVICE_URL: string;
    static DEFAULT_SERVICE_NAME: string;
    /*************************
     * Factory method
     ************************/
    /**
     * Constructs an instance of SqlV2 with passed in options and external configuration.
     *
     * @param {UserOptions} [options] - The parameters to send to the service.
     * @param {string} [options.serviceName] - The name of the service to configure
     * @param {Authenticator} [options.authenticator] - The Authenticator object used to authenticate requests to the service
     * @param {string} [options.serviceUrl] - The URL for the service
     * @returns {SqlV2}
     */
    static newInstance(options: UserOptions): SqlV2;
    /** The cloud resource name (CRN) of the SQL query service instance. */
    instanceCrn: string;
    /**
     * Construct a SqlV2 object.
     *
     * @param {Object} options - Options for the service.
     * @param {string} options.instanceCrn - The cloud resource name (CRN) of the SQL query service instance.
     * @param {string} [options.serviceUrl] - The base url to use when contacting the service. The base url may differ between IBM Cloud regions.
     * @param {OutgoingHttpHeaders} [options.headers] - Default headers that shall be included with every request to the service.
     * @param {Authenticator} options.authenticator - The Authenticator object used to authenticate requests to the service
     * @constructor
     * @returns {SqlV2}
     */
    constructor(options: UserOptions);
    /*************************
     * catalog
     ************************/
    /**
     * List catalog tables.
     *
     * Retrieve a list of the first 100 tables that are defined for the given instance in the catalog.
     *
     * @param {Object} [params] - The parameters to send to the service.
     * @param {OutgoingHttpHeaders} [params.headers] - Custom request headers
     * @returns {Promise<SqlV2.Response<SqlV2.TableList>>}
     */
    listTables(params?: SqlV2.ListTablesParams): Promise<SqlV2.Response<SqlV2.TableList>>;
    /**
     * Get information about a specific catalog table.
     *
     * Returns information about the specified catalog table.
     *
     * @param {Object} params - The parameters to send to the service.
     * @param {string} params.tableName - Name of the catalog table for which information is to be retrieved. Table names
     * are case-insensitive and must only contain alphabetic and numeral characters, and underscore (_).
     * @param {OutgoingHttpHeaders} [params.headers] - Custom request headers
     * @returns {Promise<SqlV2.Response<SqlV2.TableInformation>>}
     */
    getTable(params: SqlV2.GetTableParams): Promise<SqlV2.Response<SqlV2.TableInformation>>;
    /*************************
     * sQLJobs
     ************************/
    /**
     * Run an SQL job.
     *
     * Runs an SQL job using the *IBM Cloud SQL Query* service and stores the result in a new CSV data set in *IBM Cloud
     * Object Storage*. The `FROM` clause references rectangular data that is stored in a Parquet, CSV, or JSON file in
     * *IBM Cloud Object Storage*. Click <a
     * href="https://console.bluemix.net/docs/services/sql-query/sql-query.html#overview">here</a> for more information.
     *
     * @param {Object} params - The parameters to send to the service.
     * @param {string} params.statement - This is the SQL query to be submitted. The table names specified in its FROM
     * clause must correspond to files in one or more *IBM Cloud Object Storage* instances. The INTO clause of the query
     * indicates the endpoint, bucket, and (optionally) subfolder in *IBM Cloud Object Storage* in which the query result
     * is to be stored. Within the specified target, each result is stored in a separate subfolder with a name that
     * indicates the job ID. The job ID of a query is returned by the GET endpoint.
     * @param {string} [params.resultsetTarget] - This field provides an alternative way to specify the target URI for a
     * query. It is supported to preserve backward compatibility and will be removed in a future API version. Use the INTO
     * clause of the SQL query to specify the target URI instead.
     * @param {OutgoingHttpHeaders} [params.headers] - Custom request headers
     * @returns {Promise<SqlV2.Response<SqlV2.SqlJobInfoShort>>}
     */
    submitSqlJob(params: SqlV2.SubmitSqlJobParams): Promise<SqlV2.Response<SqlV2.SqlJobInfoShort>>;
    /**
     * Get information about recent SQL jobs.
     *
     * Returns information about recently submitted SQL jobs.
     *
     * @param {Object} [params] - The parameters to send to the service.
     * @param {OutgoingHttpHeaders} [params.headers] - Custom request headers
     * @returns {Promise<SqlV2.Response<SqlV2.SqlJobInfoList>>}
     */
    listSqlJobs(params?: SqlV2.ListSqlJobsParams): Promise<SqlV2.Response<SqlV2.SqlJobInfoList>>;
    /**
     * Get information about a specific SQL job.
     *
     * Returns information about the specified SQL job.
     *
     * @param {Object} params - The parameters to send to the service.
     * @param {string} params.jobId - ID of the SQL job for which information is to be retrieved. This ID is returned when
     * an SQL job is submitted, and when information about recently submitted SQL jobs is requested.
     * @param {OutgoingHttpHeaders} [params.headers] - Custom request headers
     * @returns {Promise<SqlV2.Response<SqlV2.SqlJobInfoFull>>}
     */
    getSqlJob(params: SqlV2.GetSqlJobParams): Promise<SqlV2.Response<SqlV2.SqlJobInfoFull>>;
}
/*************************
 * interfaces
 ************************/
declare namespace SqlV2 {
    /** Options for the `SqlV2` constructor. */
    interface Options extends UserOptions {
        /** The cloud resource name (CRN) of the SQL query service instance. */
        instanceCrn: string;
    }
    /** An operation response. */
    interface Response<T = any> {
        result: T;
        status: number;
        statusText: string;
        headers: IncomingHttpHeaders;
    }
    /** The callback for a service request. */
    type Callback<T> = (error: any, response?: Response<T>) => void;
    /** The body of a service request that returns no response data. */
    interface Empty {
    }
    /** A standard JS object, defined to avoid the limitations of `Object` and `object` */
    interface JsonObject {
        [key: string]: any;
    }
    /*************************
     * request interfaces
     ************************/
    /** Parameters for the `listTables` operation. */
    interface ListTablesParams {
        headers?: OutgoingHttpHeaders;
    }
    /** Parameters for the `getTable` operation. */
    interface GetTableParams {
        /** Name of the catalog table for which information is to be retrieved. Table names are case-insensitive and
         *  must only contain alphabetic and numeral characters, and underscore (_).
         */
        tableName: string;
        headers?: OutgoingHttpHeaders;
    }
    /** Parameters for the `submitSqlJob` operation. */
    interface SubmitSqlJobParams {
        /** This is the SQL query to be submitted. The table names specified in its FROM clause must correspond to files
         *  in one or more *IBM Cloud Object Storage* instances. The INTO clause of the query indicates the endpoint,
         *  bucket, and (optionally) subfolder in *IBM Cloud Object Storage* in which the query result is to be stored.
         *  Within the specified target, each result is stored in a separate subfolder with a name that indicates the job
         *  ID. The job ID of a query is returned by the GET endpoint.
         */
        statement: string;
        /** This field provides an alternative way to specify the target URI for a query. It is supported to preserve
         *  backward compatibility and will be removed in a future API version. Use the INTO clause of the SQL query to
         *  specify the target URI instead.
         */
        resultsetTarget?: string;
        headers?: OutgoingHttpHeaders;
    }
    /** Parameters for the `listSqlJobs` operation. */
    interface ListSqlJobsParams {
        headers?: OutgoingHttpHeaders;
    }
    /** Parameters for the `getSqlJob` operation. */
    interface GetSqlJobParams {
        /** ID of the SQL job for which information is to be retrieved. This ID is returned when an SQL job is
         *  submitted, and when information about recently submitted SQL jobs is requested.
         */
        jobId: string;
        headers?: OutgoingHttpHeaders;
    }
    /*************************
     * model interfaces
     ************************/
    /** Information about a table column. */
    interface ColumnInformation {
        /** The name of the column. */
        name: string;
        /** The data type of the column. */
        type: string;
        /** Whether the column may contain NULL values. */
        nullable?: boolean;
    }
    /** Full information about an SQL job, including output or error information. */
    interface SqlJobInfoFull {
        /** Identifier for an SQL job. */
        job_id: string;
        /** Execution status of an SQL job. */
        status: string;
        /** ID of the user who submitted an SQL job. */
        user_id: string;
        /** Timestamp indicating when an SQL job was accepted by the service. */
        submit_time: string;
        /** The SQL query that the job processes. */
        statement: string;
        /** The service plan id of the instance. */
        plan_id?: string;
        /** Format of the query result. */
        resultset_format?: string;
        /** A URI that indicates where the query result is stored. This URI can be used as input for another SQL query.
         *  The result comprises all objects that have a name with this URI as its prefix.
         */
        resultset_location?: string;
        /** Timestamp indicating when the job finished processing. */
        end_time?: string;
        /** Number of rows returned. */
        rows_returned?: number;
        /** Number of rows read. */
        rows_read?: number;
        /** Number of bytes read. */
        bytes_read?: number;
        /** Number of objects skipped using index management.(Optional). */
        objects_skipped?: number;
        /** Number of objects qualified using index management.(Optional). */
        objects_qualified?: number;
        /** An error that was encountered while processing the job. */
        error?: string;
        /** Detailed information about the error. */
        error_message?: string;
        /** Suggests possible optimizations for a query. */
        hints?: string[];
    }
    /** List of information about SQL jobs. */
    interface SqlJobInfoList {
        /** The SQL jobs. */
        jobs: SqlJobInfoShort[];
    }
    /** Abridged information about an SQL job, including its identifier and processing status. */
    interface SqlJobInfoShort {
        /** Identifier for an SQL job. */
        job_id: string;
        /** Execution status of an SQL job. */
        status: string;
        /** ID of the user who submitted an SQL job. */
        user_id?: string;
        /** Timestamp indicating when an SQL job was accepted by the service. */
        submit_time?: string;
        /** Boolean indicating when an SQL job has an improvement hint. */
        has_hints?: boolean;
    }
    /** Detailed information about a catalog table. */
    interface TableInformation {
        /** The name of a catalog table. */
        name: string;
        /** The type of a catalog table (for example, "TABLE" or "VIEW"). */
        type: string;
        /** The columns of the table. */
        columns: ColumnInformation[];
    }
    /** List of catalog tables. */
    interface TableList {
        /** The table names. */
        tables: string[];
        /** Metadata about the returned tables. */
        tables_metadata: TableMetadata[];
    }
    /** Short metadata about a catalog table. */
    interface TableMetadata {
        /** The name of a catalog table. */
        name: string;
        /** The type of a catalog table (for example, "TABLE" or "VIEW"). */
        type: string;
    }
}
export = SqlV2;
